package info.bukova.isspst;

import info.bukova.isspst.data.Role;
import info.bukova.isspst.data.User;
import info.bukova.isspst.services.RoleService;
import info.bukova.isspst.services.UserService;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

public class DbInitListener implements ServletContextListener {

	private RoleService roleService;
	private UserService userService;
	
	@Override
	public void contextDestroyed(ServletContextEvent arg0) {

	}

	@Override
	public void contextInitialized(ServletContextEvent evt) {
		Logger logger = LoggerFactory.getLogger(DbInitListener.class);
		logger.info("Initializing database");
		
		WebApplicationContext ctx = WebApplicationContextUtils.getRequiredWebApplicationContext(evt.getServletContext());
		roleService = ctx.getBean(RoleService.class);
		userService = ctx.getBean(UserService.class);
		
		checkRoles();
		checkUsers();
	}
	
	private void checkRoles() {
		Role r = null;
		
		for (String auth : Constants.ROLES)
		{
			r = roleService.getRoleByAuthority(auth);
			if (r == null) {
				r = new Role();
				r.setAuthority(auth);
				r.setDescription("---");
				roleService.add(r);
			}
		}
	}
	
	private void checkUsers() {
		boolean adminExists = false;
		
		for (User u : userService.getAll()) {
			if (userService.hasRole(u, Constants.ROLE_ADMIN)) {
				adminExists = true;
				break;
			}
		}
		
		if (!adminExists) {
			User u = new User();
			u.setUsername(Constants.DEF_ADMIN);
			u.addAuthority(roleService.getRoleByAuthority(Constants.ROLE_ADMIN));
			u.setEnabled(true);
			userService.setPassword(u, Constants.DEF_ADMIN_PASSWD);
			userService.add(u);
		}
	}

}
