package info.bukova.isspst.data;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinTable;
import javax.persistence.MapKeyColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.springframework.security.core.userdetails.UserDetails;

@Entity
@Table(name="USER")
public class User implements UserDetails, DataModel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5421234421833765433L;
	
	@Id
	@Column(name="ID")
    @GeneratedValue
	private int id;
	@Column(name="USERNAME", unique=true)
	private String username;
	@Column(name="PASSWORD")
	private String password;
	@Column(name="ENABLED")
	private boolean enabled;
	private String fullName;
	@OneToMany(fetch=FetchType.EAGER) @JoinTable(name="USER_ROLE")
	@MapKeyColumn(name="ROLE_ID")
	private List<Role> authorities;
	@Column(name="CREATED")
	private Date created;
	@Column(name="MODIFIED")
	private Date modified;
	@Transient
	private boolean valid;
	
	public User() {
		authorities = new ArrayList<Role>();
	}

	@Override
	public List<Role> getAuthorities() {
		return authorities;
	}

	@Override
	public String getPassword() {
		return password;
	}

	@Override
	public String getUsername() {
		return username;
	}

	@Override
	public boolean isAccountNonExpired() {
		return true;
	}

	@Override
	public boolean isAccountNonLocked() {
		return true;
	}

	@Override
	public boolean isCredentialsNonExpired() {
		return true;
	}

	@Override
	public boolean isEnabled() {
		return enabled;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public void setAuthorities(List<Role> authorities) {
		this.authorities = authorities;
	}
	
	public void addAuthority(Role role) {
		this.authorities.add(role);
	}

	public Date getCreated() {
		return created;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public Date getModified() {
		return modified;
	}

	public void setModified(Date modified) {
		this.modified = modified;
	}

	public boolean isValid() {
		return valid;
	}

	public void setValid(boolean valid) {
		this.valid = valid;
	}

	public String getFullName() {
		return fullName;
	}

	public void setFullName(String fullName) {
		this.fullName = fullName;
	}

}
