package info.bukova.isspst.ui;

import info.bukova.isspst.dao.IntegrityException;
import info.bukova.isspst.data.DataModel;
import info.bukova.isspst.filters.Filter;
import info.bukova.isspst.services.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.zkoss.bind.annotation.BindingParam;
import org.zkoss.bind.annotation.Command;
import org.zkoss.bind.annotation.GlobalCommand;
import org.zkoss.bind.annotation.NotifyChange;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Window;

public class ListViewModel<T extends DataModel> {
	
	private boolean confirmDelete = false;
	private boolean filter = false;
	private Window editWin;
	private T dataBean;
	//private T filterTemplate;
	private Filter<T> dataFilter;
	private T editBean;
	private List<T> dataList;
	private List<T> fullList;
	private List<T> tmpList;
	private int selIndex = -1;
	private String sortCol;
	private boolean sortDesc = true;
	private boolean newRec = false;
	private boolean fullFill = false;
		
	protected Service<T> service;
	protected Class<T> dataClass;
	protected String formZul;
	
	public List<T> getDataList() {
		if (dataList == null) {
			dataList = new ArrayList<T>();
			loadFromDbSync();
		}
		
		return dataList;
	}
	
	public void setDataBean(T data) {
		this.dataBean = data;
	}
	
	public void setDataFilter(Filter<T> dataFilter) {
		this.dataFilter = dataFilter;
	}
	
	public T getDataBean() {
		return dataBean;
	}
	
	public boolean getConfirmDelete() {
		return confirmDelete;
	}
	
	public boolean getFilter() {
		return this.filter;
	}

	public boolean getFullFill() {
		return fullFill;
	}
	
	protected void newRecMode() {
		newRec = true;
	}
	
	protected void setEditBean(T edit) {
		this.editBean = edit;
	}
	
	
	@Command
	@NotifyChange({"filter", "dataList", "dataBean"})
	public void filter() {
		filter = !filter;
		
		if (!filter) {
			dataList = fullList;
			dataBean = null;
			selIndex = -1;
		} else {
			doFilter();
			dataBean = null;
		}
	}
	
	@Command
	@NotifyChange("dataList")
	public void doFilter() {
		if (dataFilter == null) {
			return;
		}
		
		List<T> result = service.filterList(fullList, dataFilter);
		selIndex = -1;
		dataList = result;
	}
		
	@Command
	public void addNew() {
		try {
			newRecMode();
			editBean = dataClass.newInstance();
			showForm();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
	}
	
	@Command
	public void edit() {
		int index = dataList.indexOf(dataBean);
		if (index != -1) { 
			selIndex = index;
		}
		editBean = service.getById(dataBean.getId());
		showForm();
	}
	
	@Command
	@NotifyChange("confirmDelete")
	public void delObject() {
		confirmDelete = true;
	}
	
	@Command
	@NotifyChange({"dataList", "dataBean"})
	public void delete() {
		try {
			service.delete(dataBean);
			dataList.remove(dataBean);
			dataBean = null;
		} catch (IntegrityException e) {
			Messagebox.show("Error while deleting object", "Error", Messagebox.OK, Messagebox.ERROR);
		}
		confirmDelete = false;
	}
	
	@GlobalCommand
	@NotifyChange({"dataList", "dataBean"})
	public void refresh() {
		if (editBean != null && !editBean.isValid()) {
			return;
		}
		if (!filter && newRec) {
			dataList.add(editBean);
			if (dataList != fullList) {
				fullList.add(editBean);
			}
		} else if (newRec) {
			fullList.add(editBean);
		}
		dataBean = editBean;
		if (!newRec) {
			dataList.set(selIndex, editBean);
		}
	}
	
	@GlobalCommand
	@NotifyChange({"dataList", "dataBean", "fullFill"})
	public void reload() {
		dataBean = null;
		dataList.clear();
		loadFromDbSync();
	}

	@Command
	@NotifyChange("selIndex")
	public void afterRender() {
		if (editBean != null && !editBean.isValid()) {
			return;
		}
		
		if (selIndex > dataList.size() -1) {
			selIndex = -1;
		}

		if (newRec) {
			selIndex = dataList.size() -1;
			newRec = false;
		}
	}
	
	@Command
	@NotifyChange("dataBean")
	public void onSort(@BindingParam("column") String column) {
		if (sortCol == null || this.sortCol.equals(column))
			this.sortDesc = !this.sortDesc;
		
		this.sortCol = column;
		selIndex = -1;
		dataBean = null;
	}
		
	public int getSelIndex() {
		return this.selIndex;
	}
	
	@Command
	@NotifyChange({"dataList", "fullFill"})
	public void fullFill() {
		if (fullFill && dataList.isEmpty()) {
			dataList.addAll(tmpList);
			fullList = dataList;
		}
	}
	
	private void loadFromDb() {
		Thread fillThread = new Thread(new Runnable() {
			
			@Override
			public void run() {
				tmpList = service.getAll();
								
				try {
					Thread.sleep(200);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				
				fullFill = true;
			}
		});
		
		fillThread.start();
	}
	
	private void loadFromDbSync() {
		dataList.addAll(service.getAll());
	}
	
	protected void showForm() {
		Map<String, Object> arg = new HashMap<String, Object>();
		arg.put("selected", editBean);
		arg.put("service", service);
		editWin = (Window) Executions.createComponents(formZul, null, arg);
		editWin.doModal();
	}

	protected List<T> getFullList() {
		return fullList;
	}

	protected void setFullFill(boolean fullFill) {
		this.fullFill = fullFill;
	}

}
